@@ -61,11 +61,13 @@ class PlaybackStatus:
     PAUSED = 1
     STOPPED = 2
 
+
 class DiscoveryStatus:
     PENDING = 0
     FAILED = 1
     SUCCEEDED = 2
 
+
 class Player(GObject.GObject):
     nextTrack = None
     timeout = None
@@ -92,6 +94,7 @@ class Player(GObject.GObject):
         self.playlistId = None
         self.playlistField = None
         self.currentTrack = None
+        self.currentTrackUri = None
         self._lastState = Gst.State.PAUSED
         self.cache = AlbumArtCache.get_default()
         self._noArtworkIcon = self.cache.get_default_icon(ART_SIZE, ART_SIZE)
@@ -242,6 +245,8 @@ class Player(GObject.GObject):
                 currentTrack = self.playlist.get_path(self.playlist.get_iter_first())
                 if currentTrack:
                     self.currentTrack = Gtk.TreeRowReference.new(self.playlist, currentTrack)
+                    self.currentTrackUri = self.playlist.get_value(
+                        self.playlist.get_iter(self.currentTrack.get_path()), 5).get_url()
                 else:
                     self.currentTrack = None
                 self.load(self.get_current_media())
@@ -256,6 +261,8 @@ class Player(GObject.GObject):
     @log
     def _on_glib_idle(self):
         self.currentTrack = self.nextTrack
+        self.currentTrackUri = self.playlist.get_value(
+            self.playlist.get_iter(self.currentTrack.get_path()), 5).get_url()
         self.play()
 
     @log
@@ -472,12 +479,12 @@ class Player(GObject.GObject):
         self.emit('playlist-item-changed', self.playlist, currentTrack)
         self.emit('current-changed')
 
-        self._validate_next_track();
+        self._validate_next_track()
 
     def _on_next_item_validated(self, info, error, _iter):
         if error:
             print("Info %s: error: %s" % (info, error))
-            self.playlist.set_value(_iter, self.discovery_status_field, DiscoveryStatus.FAILED);
+            self.playlist.set_value(_iter, self.discovery_status_field, DiscoveryStatus.FAILED)
             nextTrack = self.playlist.iter_next(_iter)
 
             if nextTrack:
@@ -495,13 +502,17 @@ class Player(GObject.GObject):
         _iter = self.playlist.get_iter(self.nextTrack.get_path())
         status = self.playlist.get_value(_iter, self.discovery_status_field)
         nextSong = self.playlist.get_value(_iter, self.playlistField)
+        url = self.playlist.get_value(_iter, 5).get_url()
 
-        if status == DiscoveryStatus.PENDING:
+        # Skip remote tracks discovery
+        if url.startswith("http://"):
+            status = DiscoveryStatus.SUCCEEDED
+        elif status == DiscoveryStatus.PENDING:
             self.discover_item(nextSong, self._on_next_item_validated, _iter)
         elif status == DiscoveryStatus.FAILED:
-            GLib.idle_add(self._validate_next_track())
+            GLib.idle_add(self._validate_next_track)
 
-        return False;
+        return False
 
     @log
     def _on_cache_lookup(self, pixbuf, path, data=None):
@@ -560,8 +571,9 @@ class Player(GObject.GObject):
 
         self.stop()
         self.currentTrack = self.nextTrack
-
         if self.currentTrack:
+            self.currentTrackUri = self.playlist.get_value(
+                self.playlist.get_iter(self.currentTrack.get_path()), 5).get_url()
             self.play()
 
     @log
@@ -582,6 +594,8 @@ class Player(GObject.GObject):
 
         self.currentTrack = self._get_previous_track()
         if self.currentTrack:
+            self.currentTrackUri = self.playlist.get_value(
+                self.playlist.get_iter(self.currentTrack.get_path()), 5).get_url()
             self.play()
 
     @log
@@ -606,6 +620,9 @@ class Player(GObject.GObject):
         self.playlistType = type
         self.playlistId = id
         self.currentTrack = Gtk.TreeRowReference.new(model, model.get_path(iter))
+        if self.currentTrack:
+            self.currentTrackUri = self.playlist.get_value(
+                self.playlist.get_iter(self.currentTrack.get_path()), 5).get_url()
         self.playlistField = field
         self.discovery_status_field = discovery_status_field
 
