@@ -76,6 +76,11 @@ class StaticPlaylists:
         # TRANSLATORS: this is a playlist name
         TITLE = _("Favorite Songs")
 
+    @classmethod
+    def get_protected_ids(self):
+        return [str(cls.ID) for name, cls in inspect.getmembers(StaticPlaylists)
+                if inspect.isclass(cls) and not (name == "__class__")]
+
 
 class Playlists(GObject.GObject):
     __gsignals__ = {
@@ -116,11 +121,11 @@ class Playlists(GObject.GObject):
             while (cursor.next(None)):
                 playlist.ID = cursor.get_integer(1)
 
-                if not playlist.ID:
-                    # create the playlist
-                    playlist.ID = self.create_playlist_and_return_id(playlist.TITLE, playlist.TAG_TEXT)
+            if not playlist.ID:
+                # create the playlist
+                playlist.ID = self.create_playlist_and_return_id(playlist.TITLE, playlist.TAG_TEXT)
 
-                self.update_static_playlist(playlist)
+            self.update_static_playlist(playlist)
 
         for playlist in playlists:
             self.tracker.query_async(
