@@ -56,6 +56,38 @@ except Exception as e:
 playlists = Playlists.get_default()
 
 
+class StarHandler():
+    @log
+    def __init__(self, parent, star_index):
+        self.star_index = star_index
+        self.star_renderer_click = False
+        self.parent = parent
+
+    @log
+    def _add_star_renderers(self, list_widget, cols, hidden=False):
+        star_renderer = CellRendererClickablePixbuf(self.parent.view, hidden=hidden)
+        star_renderer.connect("clicked", self._on_star_toggled)
+        list_widget.add_renderer(star_renderer, lambda *args: None, None)
+        cols[0].clear_attributes(star_renderer)
+        cols[0].add_attribute(star_renderer, 'show_star', self.star_index)
+
+    @log
+    def _on_star_toggled(self, widget, path):
+        try:
+            _iter = self.parent.model.get_iter(path)
+        except TypeError:
+            return
+
+        new_value = not self.parent.model.get_value(_iter, self.star_index)
+        self.parent.model.set_value(_iter, self.star_index, new_value)
+        song_item = self.parent.model.get_value(_iter, 5)
+        grilo.toggle_favorite(song_item)  # toggle favorite status in database
+        playlists.update_static_playlist(StaticPlaylists.Favorites)
+
+        # Use this flag to ignore the upcoming _on_item_activated call
+        self.star_renderer_click = True
+
+
 class AlbumWidget(Gtk.EventBox):
 
     tracks = []
@@ -64,10 +96,11 @@ class AlbumWidget(Gtk.EventBox):
     noArtworkIcon = ALBUM_ART_CACHE.get_default_icon(256, 256, False)
 
     @log
-    def __init__(self, player):
+    def __init__(self, player, parentview):
         Gtk.EventBox.__init__(self)
         self.player = player
         self.iterToClean = None
+        self.parentview = parentview
 
         self.ui = Gtk.Builder()
         self.ui.add_from_resource('/org/gnome/Music/AlbumWidget.ui')
@@ -90,8 +123,8 @@ class AlbumWidget(Gtk.EventBox):
         self.view.remove(child_view)
         view_box.add(child_view)
         self.add(self.ui.get_object('AlbumWidget'))
+        self.star_handler = StarHandler(self, 10)
         self._add_list_renderers()
-        self.star_renderer_click = False
         self.get_style_context().add_class('view')
         self.get_style_context().add_class('content-view')
         self.show_all()
@@ -102,8 +135,8 @@ class AlbumWidget(Gtk.EventBox):
 
     @log
     def _on_item_activated(self, widget, id, path):
-        if self.star_renderer_click:
-            self.star_renderer_click = False
+        if self.star_handler.star_renderer_click:
+            self.star_handler.star_renderer_click = False
             return
 
         _iter = self.model.get_iter(path)
@@ -159,21 +192,17 @@ class AlbumWidget(Gtk.EventBox):
         cols[0].clear_attributes(durationRenderer)
         cols[0].add_attribute(durationRenderer, 'markup', 1)
 
-        star_renderer = CellRendererClickablePixbuf(self.view)
-        star_renderer.connect("clicked", self._on_star_toggled)
-        list_widget.add_renderer(star_renderer, lambda *args: None, None)
-        cols[0].clear_attributes(star_renderer)
-        cols[0].add_attribute(star_renderer, 'show_star', 10)
+        self.star_handler._add_star_renderers(list_widget, cols)
 
     def _on_list_widget_icon_render(self, col, cell, model, _iter, data):
-        if model != self.player.playlist:
+        if not self.player.currentTrackUri:
             cell.set_visible(False)
             return
 
         if model.get_value(_iter, 10) == DiscoveryStatus.FAILED:
             cell.set_property('icon-name', ERROR_ICON_NAME)
             cell.set_visible(True)
-        elif model.get_path(_iter) == self.player.currentTrack.get_path():
+        elif model.get_value(_iter, 5).get_url() == self.player.currentTrackUri:
             cell.set_property('icon-name', NOW_PLAYING_ICON_NAME)
             cell.set_visible(True)
         else:
@@ -207,18 +236,9 @@ class AlbumWidget(Gtk.EventBox):
         self.ui.get_object('cover').set_from_pixbuf(self.loadingIcon)
         ALBUM_ART_CACHE.lookup(item, 256, 256, self._on_look_up, None, real_artist, album)
 
-        # if the active queue has been set by self album,
-        # use it as model, otherwise build the liststore
-        cached_playlist = self.player.running_playlist('Album', album)
-        if cached_playlist:
-            self.model = cached_playlist
-            currentTrack = self.player.playlist.get_iter(self.player.currentTrack.get_path())
-            self.update_model(self.player, cached_playlist,
-                              currentTrack)
-        else:
-            self.duration = 0
-            self._create_model()
-            GLib.idle_add(grilo.populate_album_songs, item, self.add_item)
+        self.duration = 0
+        self._create_model()
+        GLib.idle_add(grilo.populate_album_songs, item, self.add_item)
         header_bar._select_button.connect(
             'toggled', self._on_header_select_button_toggled)
         header_bar._cancel_button.connect(
@@ -322,21 +342,21 @@ class AlbumWidget(Gtk.EventBox):
                 _("%d min") % (int(self.duration / 60) + 1))
         return False
 
-    @log
-    def _on_star_toggled(self, widget, path):
-        try:
-            _iter = self.model.get_iter(path)
-        except TypeError:
-            return
+    # @log
+    # def _on_star_toggled(self, widget, path):
+    #     try:
+    #         _iter = self.model.get_iter(path)
+    #     except TypeError:
+    #         return
 
-        new_value = not self.model.get_value(_iter, 10)
-        self.model.set_value(_iter, 10, new_value)
-        song_item = self.model.get_value(_iter, 5) # er, will this definitely return MediaAudio obj.?
-        grilo.toggle_favorite(song_item) # toggle favorite status in database
-        playlists.update_static_playlist(StaticPlaylists.Favorites)
+    #     new_value = not self.model.get_value(_iter, 10)
+    #     self.model.set_value(_iter, 10, new_value)
+    #     song_item = self.model.get_value(_iter, 5) # er, will this definitely return MediaAudio obj.?
+    #     grilo.toggle_favorite(song_item) # toggle favorite status in database
+    #     playlists.update_static_playlist(StaticPlaylists.Favorites)
 
-        # Use this flag to ignore the upcoming _on_item_activated call
-        self.star_renderer_click = True
+    #     # Use this flag to ignore the upcoming _on_item_activated call
+    #     self.star_renderer_click = True
 
 
 class ArtistAlbums(Gtk.Box):
