@@ -279,7 +279,7 @@ class Window(Gtk.ApplicationWindow):
         if self.toolbar._selectionMode is False:
             return
         if self.toolbar._state == ToolbarState.MAIN:
-            model = self._stack.get_visible_child()._model
+            model = self._stack.get_visible_child().model
         else:
             model = self._stack.get_visible_child().get_visible_child().model
         count = self._set_selection(model, True)
@@ -295,7 +295,7 @@ class Window(Gtk.ApplicationWindow):
     @log
     def _on_select_none(self, action, param):
         if self.toolbar._state == ToolbarState.MAIN:
-            model = self._stack.get_visible_child()._model
+            model = self._stack.get_visible_child().model
         else:
             model = self._stack.get_visible_child().get_visible_child().model
         self._set_selection(model, False)
@@ -304,17 +304,31 @@ class Window(Gtk.ApplicationWindow):
         self.toolbar._selection_menu_label.set_text(_("Click on items to select them"))
         self._stack.get_visible_child().queue_draw()
 
+    @log
+    def _init_loading_notification(self):
+        self.notification = Gd.Notification()
+        grid = Gtk.Grid(valign=Gtk.Align.CENTER, margin_right=8)
+        grid.set_column_spacing(8)
+        spinner = Gtk.Spinner()
+        spinner.start()
+        grid.add(spinner)
+        label = Gtk.Label.new(_("Loading"))
+        grid.add(label)
+        self.notification.add(grid)
+        self.notification.show_all()
+        GLib.timeout_add(1000, self._overlay.add_overlay, self.notification)
+
     @log
     def _init_playlist_removal_notification(self):
         self.notification = Gd.Notification()
         self.notification.set_timeout(20)
 
-        grid = Gtk.Grid()
+        grid = Gtk.Grid(valign=Gtk.Align.CENTER, margin_right=8)
         grid.set_column_spacing(8)
         self.notification.add(grid)
 
         undo_button = Gtk.Button.new_with_mnemonic(_("_Undo"))
-        label = _("Playlist %s removed" %(
+        label = _("Playlist %s removed" % (
             self.views[3].current_playlist.get_title()))
         grid.add(Gtk.Label.new(label))
         grid.add(undo_button)
@@ -322,6 +336,8 @@ class Window(Gtk.ApplicationWindow):
         self.notification.show_all()
         self._overlay.add_overlay(self.notification)
 
+        self.notification.deletion_index = self.views[3].current_playlist_index
+
         self.notification.connect("dismissed", self._playlist_removal_notification_dismissed)
         undo_button.connect("clicked", self._undo_deletion)
 
@@ -329,25 +345,14 @@ class Window(Gtk.ApplicationWindow):
     def _playlist_removal_notification_dismissed(self, widget):
         if self.views[3].really_delete:
             Views.playlists.delete_playlist(self.views[3].pl_todelete)
-
-    @log
-    def _init_loading_notification(self):
-        self.notification = Gd.Notification()
-        grid = Gtk.Grid()
-        grid.set_column_spacing(8)
-        self.notification.add(grid)
-        spinner = Gtk.Spinner()
-        grid.add(spinner)
-        grid.add(Gtk.Label.new(_("Loading")))
-        spinner.start()
-        self.notification.show_all()
-        self._overlay.add_overlay(self.notification)
+        else:
+            self.views[3].really_delete = True
 
     @log
     def _undo_deletion(self, widget):
         self.views[3].really_delete = False
         self.notification.dismiss()
-        self.views[3].undo_playlist()
+        self.views[3].undo_playlist_deletion(self.notification.deletion_index)
 
     @log
     def _on_key_press(self, widget, event):
@@ -368,8 +373,7 @@ class Window(Gtk.ApplicationWindow):
         else:
             if (event.keyval == Gdk.KEY_Delete):
                 if self._stack.get_visible_child() == self.views[3]:
-                    self._init_playlist_removal_notification()
-                    self.views[3].delete_selected_playlist()
+                    self.views[3]._on_delete_activate(None)
             # Close search bar after Esc is pressed
             if event.keyval == Gdk.KEY_Escape:
                 self.toolbar.searchbar.show_bar(False)
